#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from openstackclient.tests.functional.network.v2 import common


class NetworkRBACTests(common.NetworkTests):
    """Functional tests for network rbac"""
    OBJECT_ID = None
    ID = None
    HEADERS = ['ID']
    FIELDS = ['id']

    def setUp(self):
        super(NetworkRBACTests, self).setUp()
        # Nothing in this class works with Nova Network
        if not self.haz_network:
            self.skipTest("No Network service present")

        self.NET_NAME = uuid.uuid4().hex
        self.PROJECT_NAME = uuid.uuid4().hex

        cmd_output = json.loads(self.openstack(
            'network create -f json ' + self.NET_NAME
        ))
        self.addCleanup(self.openstack,
                        'network delete ' + cmd_output['id'])
        self.OBJECT_ID = cmd_output['id']

        cmd_output = json.loads(self.openstack(
            'network rbac create -f json ' +
            self.OBJECT_ID +
            ' --action access_as_shared' +
            ' --target-project admin' +
            ' --type network'
        ))
        self.addCleanup(self.openstack,
                        'network rbac delete ' + cmd_output['id'])
        self.ID = cmd_output['id']
        self.assertEqual(self.OBJECT_ID, cmd_output['object_id'])

    def test_network_rbac_list(self):
        cmd_output = json.loads(self.openstack('network rbac list -f json'))
        self.assertIn(self.ID, [rbac['ID'] for rbac in cmd_output])

    def test_network_rbac_show(self):
        cmd_output = json.loads(self.openstack(
            'network rbac show -f json ' + self.ID))
        self.assertEqual(self.ID, cmd_output['id'])

    def test_network_rbac_set(self):
        project_id = json.loads(self.openstack(
            'project create -f json ' + self.PROJECT_NAME))['id']
        self.openstack('network rbac set ' + self.ID +
                       ' --target-project ' + self.PROJECT_NAME)
        cmd_output_rbac = json.loads(self.openstack(
            'network rbac show -f json ' + self.ID))
        self.assertEqual(project_id, cmd_output_rbac['target_project_id'])
        raw_output_project = self.openstack(
            'project delete ' + self.PROJECT_NAME)
        self.assertEqual('', raw_output_project)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


"""Identity v3 Protocols actions implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _


LOG = logging.getLogger(__name__)


class CreateProtocol(command.ShowOne):
    _description = _("Create new federation protocol")

    def get_parser(self, prog_name):
        parser = super(CreateProtocol, self).get_parser(prog_name)
        parser.add_argument(
            'federation_protocol',
            metavar='<name>',
            help=_('New federation protocol name (must be unique '
                   'per identity provider)'),
        )
        parser.add_argument(
            '--identity-provider',
            metavar='<identity-provider>',
            required=True,
            help=_('Identity provider that will support the new federation '
                   ' protocol (name or ID) (required)'),
        )
        parser.add_argument(
            '--mapping',
            metavar='<mapping>',
            required=True,
            help=_('Mapping that is to be used (name or ID) (required)'),
        )

        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        protocol = identity_client.federation.protocols.create(
            protocol_id=parsed_args.federation_protocol,
            identity_provider=parsed_args.identity_provider,
            mapping=parsed_args.mapping)
        info = dict(protocol._info)
        # NOTE(marek-denis): Identity provider is not included in a response
        # from Keystone, however it should be listed to the user. Add it
        # manually to the output list, simply reusing value provided by the
        # user.
        info['identity_provider'] = parsed_args.identity_provider
        info['mapping'] = info.pop('mapping_id')
        info.pop('links', None)
        return zip(*sorted(info.items()))


class DeleteProtocol(command.Command):
    _description = _("Delete federation protocol(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteProtocol, self).get_parser(prog_name)
        parser.add_argument(
            'federation_protocol',
            metavar='<federation-protocol>',
            nargs='+',
            help=_('Federation protocol(s) to delete (name or ID)'),
        )
        parser.add_argument(
            '--identity-provider',
            metavar='<identity-provider>',
            required=True,
            help=_('Identity provider that supports <federation-protocol> '
                   '(name or ID) (required)'),
        )

        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        result = 0
        for i in parsed_args.federation_protocol:
            try:
                identity_client.federation.protocols.delete(
                    parsed_args.identity_provider, i)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete federation protocol "
                          "with name or ID '%(protocol)s': %(e)s"),
                          {'protocol': i, 'e': e})

        if result > 0:
            total = len(parsed_args.federation_protocol)
            msg = (_("%(result)s of %(total)s federation protocols failed"
                   " to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListProtocols(command.Lister):
    _description = _("List federation protocols")

    def get_parser(self, prog_name):
        parser = super(ListProtocols, self).get_parser(prog_name)
        parser.add_argument(
            '--identity-provider',
            metavar='<identity-provider>',
            required=True,
            help=_('Identity provider to list (name or ID) (required)'),
        )

        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        protocols = identity_client.federation.protocols.list(
            parsed_args.identity_provider)
        columns = ('id', 'mapping')
        response_attributes = ('id', 'mapping_id')
        items = [utils.get_item_properties(s, response_attributes)
                 for s in protocols]
        return (columns, items)


class SetProtocol(command.Command):
    _description = _("Set federation protocol properties")

    def get_parser(self, prog_name):
        parser = super(SetProtocol, self).get_parser(prog_name)
        parser.add_argument(
            'federation_protocol',
            metavar='<name>',
            help=_('Federation protocol to modify (name or ID)'),
        )
        parser.add_argument(
            '--identity-provider',
            metavar='<identity-provider>',
            required=True,
            help=_('Identity provider that supports <federation-protocol> '
                   '(name or ID) (required)'),
        )
        parser.add_argument(
            '--mapping',
            metavar='<mapping>',
            help=_('Mapping that is to be used (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        protocol = identity_client.federation.protocols.update(
            parsed_args.identity_provider, parsed_args.federation_protocol,
            parsed_args.mapping)
        info = dict(protocol._info)
        # NOTE(marek-denis): Identity provider is not included in a response
        # from Keystone, however it should be listed to the user. Add it
        # manually to the output list, simply reusing value provided by the
        # user.
        info['identity_provider'] = parsed_args.identity_provider
        info['mapping'] = info.pop('mapping_id')
        return zip(*sorted(info.items()))


class ShowProtocol(command.ShowOne):
    _description = _("Display federation protocol details")

    def get_parser(self, prog_name):
        parser = super(ShowProtocol, self).get_parser(prog_name)
        parser.add_argument(
            'federation_protocol',
            metavar='<federation-protocol>',
            help=_('Federation protocol to display (name or ID)'),
        )
        parser.add_argument(
            '--identity-provider',
            metavar='<identity-provider>',
            required=True,
            help=_('Identity provider that supports <federation-protocol> '
                   '(name or ID) (required)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        protocol = identity_client.federation.protocols.get(
            parsed_args.identity_provider, parsed_args.federation_protocol)
        info = dict(protocol._info)
        info['mapping'] = info.pop('mapping_id')
        info.pop('links', None)
        return zip(*sorted(info.items()))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from novaclient import api_versions
from osc_lib import exceptions

from openstackclient.compute.v2 import server_volume
from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes


class TestServerVolume(compute_fakes.TestComputev2):

    def setUp(self):
        super().setUp()

        # Get a shortcut to the compute client ServerManager Mock
        self.servers_mock = self.app.client_manager.compute.servers
        self.servers_mock.reset_mock()

        # Get a shortcut to the compute client VolumeManager mock
        self.servers_volumes_mock = self.app.client_manager.compute.volumes
        self.servers_volumes_mock.reset_mock()


class TestServerVolumeList(TestServerVolume):

    def setUp(self):
        super().setUp()

        self.server = compute_fakes.FakeServer.create_one_server()
        self.volume_attachments = (
            compute_fakes.FakeVolumeAttachment.create_volume_attachments())

        self.servers_mock.get.return_value = self.server
        self.servers_volumes_mock.get_server_volumes.return_value = (
            self.volume_attachments)

        # Get the command object to test
        self.cmd = server_volume.ListServerVolume(self.app, None)

    def test_server_volume_list(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.1')

        arglist = [
            self.server.id,
        ]
        verifylist = [
            ('server', self.server.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(('ID', 'Device', 'Server ID', 'Volume ID'), columns)
        self.assertEqual(
            (
                (
                    self.volume_attachments[0].id,
                    self.volume_attachments[0].device,
                    self.volume_attachments[0].serverId,
                    self.volume_attachments[0].volumeId,
                ),
                (
                    self.volume_attachments[1].id,
                    self.volume_attachments[1].device,
                    self.volume_attachments[1].serverId,
                    self.volume_attachments[1].volumeId,
                ),
            ),
            tuple(data),
        )
        self.servers_volumes_mock.get_server_volumes.assert_called_once_with(
            self.server.id)

    def test_server_volume_list_with_tags(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.70')

        arglist = [
            self.server.id,
        ]
        verifylist = [
            ('server', self.server.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(
            ('ID', 'Device', 'Server ID', 'Volume ID', 'Tag',), columns,
        )
        self.assertEqual(
            (
                (
                    self.volume_attachments[0].id,
                    self.volume_attachments[0].device,
                    self.volume_attachments[0].serverId,
                    self.volume_attachments[0].volumeId,
                    self.volume_attachments[0].tag,
                ),
                (
                    self.volume_attachments[1].id,
                    self.volume_attachments[1].device,
                    self.volume_attachments[1].serverId,
                    self.volume_attachments[1].volumeId,
                    self.volume_attachments[1].tag,
                ),
            ),
            tuple(data),
        )
        self.servers_volumes_mock.get_server_volumes.assert_called_once_with(
            self.server.id)

    def test_server_volume_list_with_delete_on_attachment(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.79')

        arglist = [
            self.server.id,
        ]
        verifylist = [
            ('server', self.server.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(
            (
                'ID', 'Device', 'Server ID', 'Volume ID', 'Tag',
                'Delete On Termination?',
            ),
            columns,
        )
        self.assertEqual(
            (
                (
                    self.volume_attachments[0].id,
                    self.volume_attachments[0].device,
                    self.volume_attachments[0].serverId,
                    self.volume_attachments[0].volumeId,
                    self.volume_attachments[0].tag,
                    self.volume_attachments[0].delete_on_termination,
                ),
                (
                    self.volume_attachments[1].id,
                    self.volume_attachments[1].device,
                    self.volume_attachments[1].serverId,
                    self.volume_attachments[1].volumeId,
                    self.volume_attachments[1].tag,
                    self.volume_attachments[1].delete_on_termination,
                ),
            ),
            tuple(data),
        )
        self.servers_volumes_mock.get_server_volumes.assert_called_once_with(
            self.server.id)

    def test_server_volume_list_with_attachment_ids(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.89')

        arglist = [
            self.server.id,
        ]
        verifylist = [
            ('server', self.server.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(
            (
                'Device', 'Server ID', 'Volume ID', 'Tag',
                'Delete On Termination?', 'Attachment ID',
                'BlockDeviceMapping UUID',
            ),
            columns,
        )
        self.assertEqual(
            (
                (
                    self.volume_attachments[0].device,
                    self.volume_attachments[0].serverId,
                    self.volume_attachments[0].volumeId,
                    self.volume_attachments[0].tag,
                    self.volume_attachments[0].delete_on_termination,
                    self.volume_attachments[0].attachment_id,
                    self.volume_attachments[0].bdm_uuid

                ),
                (
                    self.volume_attachments[1].device,
                    self.volume_attachments[1].serverId,
                    self.volume_attachments[1].volumeId,
                    self.volume_attachments[1].tag,
                    self.volume_attachments[1].delete_on_termination,
                    self.volume_attachments[1].attachment_id,
                    self.volume_attachments[1].bdm_uuid
                ),
            ),
            tuple(data),
        )
        self.servers_volumes_mock.get_server_volumes.assert_called_once_with(
            self.server.id)


class TestServerVolumeUpdate(TestServerVolume):

    def setUp(self):
        super().setUp()

        self.server = compute_fakes.FakeServer.create_one_server()
        self.servers_mock.get.return_value = self.server

        # Get the command object to test
        self.cmd = server_volume.UpdateServerVolume(self.app, None)

    def test_server_volume_update(self):

        arglist = [
            self.server.id,
            'foo',
        ]
        verifylist = [
            ('server', self.server.id),
            ('volume', 'foo'),
            ('delete_on_termination', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # This is a no-op
        self.servers_volumes_mock.update_server_volume.assert_not_called()
        self.assertIsNone(result)

    def test_server_volume_update_with_delete_on_termination(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.85')

        arglist = [
            self.server.id,
            'foo',
            '--delete-on-termination',
        ]
        verifylist = [
            ('server', self.server.id),
            ('volume', 'foo'),
            ('delete_on_termination', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_volumes_mock.update_server_volume.assert_called_once_with(
            self.server.id, 'foo', 'foo',
            delete_on_termination=True)
        self.assertIsNone(result)

    def test_server_volume_update_with_preserve_on_termination(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.85')

        arglist = [
            self.server.id,
            'foo',
            '--preserve-on-termination',
        ]
        verifylist = [
            ('server', self.server.id),
            ('volume', 'foo'),
            ('delete_on_termination', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.servers_volumes_mock.update_server_volume.assert_called_once_with(
            self.server.id, 'foo', 'foo',
            delete_on_termination=False)
        self.assertIsNone(result)

    def test_server_volume_update_with_delete_on_termination_pre_v285(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.84')

        arglist = [
            self.server.id,
            'foo',
            '--delete-on-termination',
        ]
        verifylist = [
            ('server', self.server.id),
            ('volume', 'foo'),
            ('delete_on_termination', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_server_volume_update_with_preserve_on_termination_pre_v285(self):
        self.app.client_manager.compute.api_version = \
            api_versions.APIVersion('2.84')

        arglist = [
            self.server.id,
            'foo',
            '--preserve-on-termination',
        ]
        verifylist = [
            ('server', self.server.id),
            ('volume', 'foo'),
            ('delete_on_termination', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)



#   Copyright 2021 Huawei, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Node Local IP action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common as identity_common

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden_columns = ['location', 'name', 'id']
    return utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden_columns)


class CreateLocalIPAssociation(command.ShowOne):
    _description = _("Create Local IP Association")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'local_ip',
            metavar='<local-ip>',
            help=_("Local IP that the port association belongs to "
                   "(IP address or ID)")
        )
        parser.add_argument(
            'fixed_port',
            metavar='<fixed_port>',
            help=_("The ID or Name of Port to allocate Local IP Association")
        )
        parser.add_argument(
            '--fixed-ip',
            metavar='<fixed_ip>',
            help=_("Fixed IP for Local IP Association")
        )

        identity_common.add_project_domain_option_to_parser(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network

        attrs = {}
        port = client.find_port(parsed_args.fixed_port,
                                ignore_missing=False)
        attrs['fixed_port_id'] = port.id
        if parsed_args.fixed_ip:
            attrs['fixed_ip'] = parsed_args.fixed_ip
        local_ip = client.find_local_ip(
            parsed_args.local_ip,
            ignore_missing=False,
        )
        obj = client.create_local_ip_association(local_ip.id, **attrs)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})

        return (display_columns, data)


class DeleteLocalIPAssociation(command.Command):
    _description = _("Delete Local IP association(s)")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)
        parser.add_argument(
            'local_ip',
            metavar="<local_ip>",
            help=_("Local IP that the port association belongs to "
                   "(IP address or ID)")
        )
        parser.add_argument(
            'fixed_port_id',
            nargs="+",
            metavar="<fixed_port_id>",
            help=_("The fixed port ID of Local IP Association")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        local_ip = client.find_local_ip(
            parsed_args.local_ip,
            ignore_missing=False,
        )
        result = 0

        for fixed_port_id in parsed_args.fixed_port_id:
            try:
                client.delete_local_ip_association(
                    local_ip.id,
                    fixed_port_id,
                    ignore_missing=False,
                )
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete Local IP Association with "
                            "fixed port "
                            "name or ID '%(fixed_port_id)s': %(e)s"),
                          {'fixed port ID': fixed_port_id, 'e': e})

        if result > 0:
            total = len(parsed_args.fixed_port_id)
            msg = (_("%(result)s of %(total)s Local IP Associations failed "
                     "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListLocalIPAssociation(command.Lister):
    _description = _("List Local IP Associations")

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'local_ip',
            metavar='<local_ip>',
            help=_("Local IP that port associations belongs to")
        )
        parser.add_argument(
            '--fixed-port',
            metavar='<fixed_port>',
            help=_("Filter the list result by the ID or name of "
                   "the fixed port")
        )
        parser.add_argument(
            '--fixed-ip',
            metavar='<fixed_ip>',
            help=_("Filter the list result by fixed ip")
        )
        parser.add_argument(
            '--host',
            metavar='<host>',
            help=_("Filter the list result by given host")
        )
        identity_common.add_project_domain_option_to_parser(parser)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.network
        columns = (
            'local_ip_id',
            'local_ip_address',
            'fixed_port_id',
            'fixed_ip',
            'host',
        )
        column_headers = (
            'Local IP ID',
            'Local IP Address',
            'Fixed port ID',
            'Fixed IP',
            'Host'
        )
        attrs = {}
        obj = client.find_local_ip(
            parsed_args.local_ip,
            ignore_missing=False,
        )
        if parsed_args.fixed_port:
            port = client.find_port(parsed_args.fixed_port,
                                    ignore_missing=False)
            attrs['fixed_port_id'] = port.id
        if parsed_args.fixed_ip:
            attrs['fixed_ip'] = parsed_args.fixed_ip
        if parsed_args.host:
            attrs['host'] = parsed_args.host

        data = client.local_ip_associations(obj, **attrs)

        return (column_headers,
                (utils.get_item_properties(s,
                                           columns,
                                           formatters={}) for s in data))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from tempest.lib import exceptions

from openstackclient.tests.functional import base


class QuotaTests(base.TestCase):
    """Functional tests for quota

    Note that for 'set' tests use different quotas for each API in different
    test runs as these may run in parallel and otherwise step on each other.
    """

    PROJECT_NAME = None

    @classmethod
    def setUpClass(cls):
        super(QuotaTests, cls).setUpClass()
        cls.haz_network = cls.is_service_enabled('network')
        cls.PROJECT_NAME =\
            cls.get_openstack_configuration_value('auth.project_name')

    def test_quota_list_details_compute(self):
        expected_headers = ["Resource", "In Use", "Reserved", "Limit"]
        cmd_output = json.loads(self.openstack(
            'quota list -f json --detail --compute'
        ))
        self.assertIsNotNone(cmd_output)
        resources = []
        for row in cmd_output:
            row_headers = [str(r) for r in row.keys()]
            self.assertEqual(sorted(expected_headers), sorted(row_headers))
            resources.append(row['Resource'])
        # Ensure that returned quota is compute quota
        self.assertIn("instances", resources)
        # and that there is no network quota here
        self.assertNotIn("networks", resources)

    def test_quota_list_details_network(self):
        expected_headers = ["Resource", "In Use", "Reserved", "Limit"]
        cmd_output = json.loads(self.openstack(
            'quota list -f json --detail --network'
        ))
        self.assertIsNotNone(cmd_output)
        resources = []
        for row in cmd_output:
            row_headers = [str(r) for r in row.keys()]
            self.assertEqual(sorted(expected_headers), sorted(row_headers))
            resources.append(row['Resource'])
        # Ensure that returned quota is network quota
        self.assertIn("networks", resources)
        # and that there is no compute quota here
        self.assertNotIn("instances", resources)

    def test_quota_list_network_option(self):
        if not self.haz_network:
            self.skipTest("No Network service present")
        self.openstack('quota set --networks 40 ' + self.PROJECT_NAME)
        cmd_output = json.loads(self.openstack(
            'quota list -f json --network'
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(
            40,
            cmd_output[0]["Networks"],
        )

    def test_quota_list_compute_option(self):
        self.openstack('quota set --instances 30 ' + self.PROJECT_NAME)
        cmd_output = json.loads(self.openstack(
            'quota list -f json --compute'
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(
            30,
            cmd_output[0]["Instances"],
        )

    def test_quota_list_volume_option(self):
        self.openstack('quota set --volumes 20 ' + self.PROJECT_NAME)
        cmd_output = json.loads(self.openstack(
            'quota list -f json --volume'
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(
            20,
            cmd_output[0]["Volumes"],
        )

    def test_quota_set_project(self):
        """Test quota set, show"""
        network_option = ""
        if self.haz_network:
            network_option = "--routers 21 "
        self.openstack(
            'quota set --cores 31 --backups 41 ' +
            network_option +
            self.PROJECT_NAME
        )
        cmd_output = json.loads(self.openstack(
            'quota show -f json ' + self.PROJECT_NAME
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(
            31,
            cmd_output["cores"],
        )
        self.assertEqual(
            41,
            cmd_output["backups"],
        )
        if self.haz_network:
            self.assertEqual(
                21,
                cmd_output["routers"],
            )

        # Check default quotas
        cmd_output = json.loads(self.openstack(
            'quota show -f json --default'
        ))
        self.assertIsNotNone(cmd_output)
        # We don't necessarily know the default quotas, we're checking the
        # returned attributes
        self.assertTrue(cmd_output["cores"] >= 0)
        self.assertTrue(cmd_output["backups"] >= 0)
        if self.haz_network:
            self.assertTrue(cmd_output["routers"] >= 0)

    def test_quota_set_class(self):
        self.openstack(
            'quota set --key-pairs 33 --snapshots 43 ' +
            '--class default'
        )
        cmd_output = json.loads(self.openstack(
            'quota show -f json --class default'
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(
            33,
            cmd_output["key-pairs"],
        )
        self.assertEqual(
            43,
            cmd_output["snapshots"],
        )

        # Check default quota class
        cmd_output = json.loads(self.openstack(
            'quota show -f json --class'
        ))
        self.assertIsNotNone(cmd_output)
        # We don't necessarily know the default quotas, we're checking the
        # returned attributes
        self.assertTrue(cmd_output["key-pairs"] >= 0)
        self.assertTrue(cmd_output["snapshots"] >= 0)

    def test_quota_network_set_with_check_limit(self):
        if not self.haz_network:
            self.skipTest('No Network service present')
        if not self.is_extension_enabled('quota-check-limit'):
            self.skipTest('No "quota-check-limit" extension present')

        self.openstack('quota set --networks 40 ' + self.PROJECT_NAME)
        cmd_output = json.loads(self.openstack(
            'quota list -f json --network'
        ))
        self.assertIsNotNone(cmd_output)
        self.assertEqual(40, cmd_output[0]['Networks'])

        # That will ensure we have at least two networks in the system.
        for _ in range(2):
            self.openstack('network create --project %s %s' %
                           (self.PROJECT_NAME, uuid.uuid4().hex))

        self.assertRaises(exceptions.CommandFailed, self.openstack,
                          'quota set --networks 1 --check-limit ' +
                          self.PROJECT_NAME)
